--[[
	Adds a new tab to the auction house for more convinient selling of items.
	Enables the selling of many items at once, prefills the desired prizes and
	remembers them.
	
	Copyright (C) Udorn (Blackhand)
	
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.	
--]]

vendor.Seller = vendor.Vendor:NewModule("Seller", "AceEvent-3.0", "AceHook-3.0");
local L = vendor.Locale.GetInstance()
local self = vendor.Seller;
local log = vendor.Debug:new("Seller")

vendor.Seller.PRIZE_MODEL_FIX = 1
vendor.Seller.PRIZE_MODEL_MARKET = 2
vendor.Seller.PRIZE_MODEL_CURRENT = 3
vendor.Seller.PRIZE_MODEL_UNDERCUT = 4
vendor.Seller.PRIZE_MODEL_MULTIPLIER = 5
vendor.Seller.PRIZE_MODEL_LOWER = 6

local SELLER_TAB = "seller"
local INVENTORY_LINE_HEIGHT = 27
local INVENTORY_LINES = 11
local BID_TYPE_PER_ITEM = 1
local BID_TYPE_PER_STACK = 2
local BID_TYPE_ALL = 3
local DURATIONS = {720, 1440, 2880}
local STACK_DROPDOWN_ID = 1
local COUNT_DROPDOWN_ID = 2
local PRIZING_DROPDOWN_ID = 3
local DURATION_DROPDOWN_ID = 5
local MARKET_PERCENT = 2
local SELLER_VERSION = 5
local BID_TYPES = {L["Per item"], L["Stack"], L["Overall"]} 

--[[
	Migrates the database.
--]]
local function _MigrateDb(self)
	if (not self.db.profile.version or self.db.profile.version < 5) then
		self.db.profile.buyoutMod = {}
		self.db.profile.buyoutMod[self.PRIZE_MODEL_MARKET] = {percent = self.db.profile.marketBuyoutMul or 150, modType = 1}
		self.db.profile.buyoutMod[self.PRIZE_MODEL_CURRENT] = {percent = self.db.profile.currentBuyoutMul or 100, modType = 1}
		self.db.profile.buyoutMod[self.PRIZE_MODEL_UNDERCUT] = {percent = self.db.profile.undercutBuyoutMul or 99, modType = 1}
		self.db.profile.buyoutMod[self.PRIZE_MODEL_LOWER] = {percent = self.db.profile.lowerBuyoutMul or 99, modType = 1}
		self.db.profile.buyoutMod[self.PRIZE_MODEL_MULTIPLIER] = {percent = self.db.profile.multiplierBuyoutMul or 250, modType = 1}
		self.db.profile.defaultDuration = 3
		self.db.profile.marketBuyoutMul = nil
		self.db.profile.currentBuyoutMul = nil
		self.db.profile.undercutBuyoutMul = nil
		self.db.profile.lowerBuyoutMul = nil
		self.db.profile.multiplierBuyoutMul = nil
	end	
	self.db.profile.version = SELLER_VERSION
end
    	
--[[
	Updates some fonts.
--]]
local function _UpdateFonts(self)
	if (self.buyoutFont) then
		self.buyoutFont:SetText(L["Buyout Price"].." |cff808080("..BID_TYPES[self.db.profile.bidType]..")|r")
	end
	if (self.minBidFont) then
		self.minBidFont:SetText(L["Starting Price"].." |cff808080("..BID_TYPES[self.db.profile.bidType]..")|r")
	end
end
	
--[[
	The refresh button has been clicked. A scan will be triggered.
--]]
local function _RefreshClicked(self) 
	if (self.item) then
		vendor.Scanner:Scan(self.item.link);
	elseif (self.lastItem) then
		vendor.Scanner:Scan(self.lastItem.link);
	end
end

--[[
	Cancels the selected list of items.
--]]
local function _CancelAuctions(self)
	local auctions = {}
	for row in pairs(self.itemModel:GetSelectedItems()) do
		local itemLinkKey, _, _, count, minBid, _, buyout = self.itemModel:Get(row)
		table.insert(auctions, {itemLinkKey = itemLinkKey, count = count, minBid = minBid, buyout = buyout})
	end
	vendor.AuctionHouse:CancelAuctions(auctions, function(arg1) _RefreshClicked(arg1) end, self)
end

--[[
	Bids on the selected list of items.
--]]
local function _Bid(self)
	local auctions = {}
	for row in pairs(self.itemModel:GetSelectedItems()) do
		local itemLinkKey, _, _, count, minBid, _, buyout = self.itemModel:Get(row)
		table.insert(auctions, {itemLinkKey = itemLinkKey, count = count, minBid = minBid, buyout = 0})
	end
	vendor.Sniper:BidOrBuyAuctions(auctions)
end

--[[
	Buys the selected list of items.
--]]
local function _Buyout(self)
	local auctions = {}
	for row in pairs(self.itemModel:GetSelectedItems()) do
		local itemLinkKey, _, _, count, minBid, _, buyout = self.itemModel:Get(row)
		table.insert(auctions, {itemLinkKey = itemLinkKey, count = count, minBid = 0, buyout = buyout})
	end
	vendor.Sniper:BidOrBuyAuctions(auctions)
end

local CMDS = {
	refresh = {
		title = L["Refresh"],
		alignLeft = true,
		arg1 = vendor.Seller,
		func = function(arg1) _RefreshClicked(arg1) end,
		enabledFunc = function(self)
			return self.itemModel:GetScanSet()
		end,
		order = 9
	},
	bid = {
		title = L["Bid"],
		arg1 = vendor.Seller,
		func = function(arg1) _Bid(arg1) end,
		enabledFunc = function(self)
			local rtn = false
			for row in pairs(self.itemModel:GetSelectedItems()) do
				rtn = true
				local _, _, _, _, _, _, _, _, owner = self.itemModel:Get(row)
				if (self.playerName == owner) then
					rtn = false
					break
				end
			end
			return rtn
		end,
		order = 10
	},
	buy = {
		title = L["Buyout"],
		arg1 = vendor.Seller,
		func = function(arg1) _Buyout(arg1) end,
		enabledFunc = function(self)
			local rtn = false
			for row in pairs(self.itemModel:GetSelectedItems()) do
				rtn = true
				local _, _, _, _, _, _, buyoutPrice, _, owner = self.itemModel:Get(row)
				if (not buyoutPrice or buyoutPrice == 0 or self.playerName == owner) then
					rtn = false
					break
				end
			end
			return rtn
		end,
		order = 11
	},
	cancel = {
		title = L["Cancel Auction"],
		arg1 = vendor.Seller,
		width = 125,
		func = function(arg1) _CancelAuctions(self) end,
		enabledFunc = function(self)
			local rtn = false
			for row in pairs(self.itemModel:GetSelectedItems()) do
				rtn = true
				local _, _, _, _, _, _, _, _, owner = self.itemModel:Get(row)
				if (not owner or owner ~= self.playerName) then
					rtn = false
					break
				end
			end
			return rtn
		end,
		order = 12
	}
}

--[[
	Tries to retrieve a link for an item with the given name in the inventory.
--]]
local function _FindInventoryItemLink(name)
   	for bag = 0, 4 do
      	for slot = 1, GetContainerNumSlots(bag) do
	 		local itemLink = GetContainerItemLink(bag, slot);
	 		if (itemLink) then
	    		local itemName = GetItemInfo(itemLink);
	    		if (name and name == itemName) then
	    			return itemLink; -- don't like this, but we need to get out of two loops
	    		end
	 		end
      	end
   	end
   	return nil;
end

--[[
	Returns "minBid, buyOut" for the given item, if known.
--]]
local function _GetItemAuctionSellValues(itemLink)
	local itemLinkKey = vendor.Items:GetItemLinkKey(itemLink);
	local itemInfo = vendor.Items:GetItemInfo(itemLinkKey, vendor.AuctionHouse:IsNeutral());
	if (itemInfo) then
		return itemInfo.minBid, itemInfo.buyout;
	end
	return nil;
end

--[[
	Returns "avgMinBid, avgBuyout" for the given item, if known.
--]] 
local function _GetItemAuctionAvgValues(itemLink)
	local itemLinkKey = vendor.Items:GetItemLinkKey(itemLink);
	local itemInfo = vendor.Items:GetItemInfo(itemLinkKey, vendor.AuctionHouse:IsNeutral());
	if (itemInfo) then
		return itemInfo.avgMinBid, itemInfo.avgBuyout;
	end
	return nil;
end

--[[
	Modifies the given minBid and buyout prices by the given modifier.
--]]
local function _ModifyPrices(minBid, buyout)
	log:Debug("ModifyPrices minBid: %d buyout: %d", minBid or 0, buyout or 0)
	if (minBid and minBid > 0) then
		minBid = math.max(1, minBid)
		if (self.db.profile.bidCalc) then
			minBid = self.buyoutMul:ModifyBuyout(minBid)
		end
		if (buyout and buyout > 0) then
			buyout = self.buyoutMul:ModifyBuyout(buyout)
		end
	end
	if (not self.db.profile.bidCalc and buyout and buyout > 0) then
		minBid = math.floor(buyout * (self.db.profile.bidMul / 100.0) + 0.5)
		minBid = math.max(1, minBid)
	end
	if (self.startPrize and (not minBid or (self.startPrize > minBid))) then
		minBid = self.startPrize
	end
	if (minBid and buyout and buyout > 0 and minBid > buyout) then
		buyout = minBid; 
	end
	log:Debug("ModifyPrices exit minBid: %d buyout: %d", minBid or 0, buyout or 0)
	return minBid, buyout;	
end

--[[
	Returns item's current price calculation for current auctions.
--]] 
local function _GetCurrentPrice(self, itemLink)
	local minBid, buyout = vendor.Statistic:GetCurrentAuctionInfo(itemLink, vendor.AuctionHouse:IsNeutral())
	return _ModifyPrices(minBid, buyout)
end

--[[
	Returns item's undercut calculation for current auctions.
--]] 
local function _GetUndercutPrice(self, itemLink)
	local _, _, minBid, buyout = vendor.Statistic:GetCurrentAuctionInfo(itemLink, vendor.AuctionHouse:IsNeutral())
	return _ModifyPrices(minBid, buyout)
end

--[[
	Returns item's mulltiplier calculation for current auctions.
--]] 
local function _GetMultiplierPrice(self, itemLink)
	local minBid = self.startPrize
	local buyout = self.startPrize
	return _ModifyPrices(minBid, buyout)
end

--[[
	Returns item's market price calculation for current auctions.
--]] 
local function _GetMarketPrice(self, itemLink)
	local minBid, buyout = _GetItemAuctionAvgValues(itemLink)
	return _ModifyPrices(minBid, buyout)
end

--[[
	Returns item's lower market price calculation for current auctions.
--]] 
local function _GetLowerPrice(self, itemLink)
	local result = vendor.Scanner:GetScanSet(vendor.AuctionHouse:IsNeutral(), itemLink)
	local minBid = nil
	local buyout = nil
	if (result and result:Size() > 0) then
		-- search for first auction above lower market threshold
		local avgMinBid, avgBuyout = _GetItemAuctionAvgValues(itemLink)
		log:Debug("_GetLowerPrice avgMinBid: %d avgBuyout: %d", avgMinBid or 0, avgBuyout or 0)
		if (avgBuyout and avgBuyout > 0) then
			local limit = avgBuyout * (self.db.profile.lowerMarketThreshold / 100.0)
			log:Debug("_GetLowerPrice limit: %d", limit)
			minBid = limit	
    		for i = 1, result:Size() do
    			local itemLinkKey, time, timeLeft, count, m, minIncrement, b = result:Get(i)
    			if (itemLinkKey) then
    				if (b and b > 0 and b > limit) then
    					if (not buyout or buyout > b) then
    						buyout = b
    					end 
    				end
    			end
    		end
		end
	end
	return _ModifyPrices(minBid, buyout)
end

--[[
	Sets the given prizes for the item to be sold. 
--]]
local function _SetPrizes(self, minBid, buyout)
	MoneyInputFrame_SetCopper(self.startPriceBut, minBid);
	MoneyInputFrame_SetCopper(self.buyoutPriceBut, buyout);
end

--[[
	Update the prices and sell info display.
--]]
local function _UpdatePrices(self)
	if (not self.item) then
		self.sellInfo:Clear()
	else
    	local stackSize = self.stackDropDown:GetSelectedValue()
    	local deposit = 0
    	local perItem = self.buyoutPerItem or 0
    	local perStack = perItem * stackSize
    	local overall = perItem * self.item.count
    	-- prices
    	if (self.db.profile.bidType == BID_TYPE_PER_ITEM) then
    		_SetPrizes(self, self.minBidPerItem, perItem);
    	elseif (self.db.profile.bidType == BID_TYPE_PER_STACK) then
    		_SetPrizes(self, self.minBidPerItem * stackSize, perStack);
    	elseif (self.db.profile.bidType == BID_TYPE_ALL) then
    		_SetPrizes(self, self.minBidPerItem * self.item.count, overall);
    	end
    	-- deposit
		deposit = self.deposits[self.duration]
		local stackCount = self.countDropDown:GetSelectedValue()
		if (type(stackCount) == "string") then
			deposit = deposit * self.item.count
		else
			deposit = deposit * stackSize * stackCount
		end
    	-- sell info
    	if (self.db.profile.bidType == BID_TYPE_PER_ITEM) then
			perItem = nil
    	elseif (self.db.profile.bidType == BID_TYPE_PER_STACK) then
			perStack = nil
    	elseif (self.db.profile.bidType == BID_TYPE_ALL) then
    		overall = nil
    	end
    	self.sellInfo:SetInfo(deposit, perItem, perStack, overall)
    end
end

--[[
	Updates the count dropdown button
--]]
local function _UpdateCountDropDown(self)
	if (not self.item) then
		return;
	end
	local counts = {};
	local stackCount = self.stackDropDown:GetSelectedValue();
	local n = self.item.count / stackCount;
	if (n < 1) then
		n = 1;
	end
	n = math.floor(n);
	local hasAdd = self.item.count > n * stackCount;
	local select = n;
	if (hasAdd) then
		select = ""..n.."+";
	end
	local i = 1;
	local prev = nil;
	-- add all integral stack sizes
	while i <= n do
		table.insert(counts, i);
		prev = i;
		i = i * 2;
	end
	-- add the last if there are more items
	if (hasAdd or prev and prev < n) then
		if (hasAdd) then
			table.insert(counts, select);
		else
			table.insert(counts, n);
		end
	end
	if (type(select) == "string") then
		self.countDropDown:SetRange(1, n, {[select] = true})
	else
		self.countDropDown:SetRange(1, n)
	end
	self.countDropDown:SetItems(counts, select)
end

--[[
	Updates the dropdown buttons accoring to the selected item
--]]
local function _UpdateDropDowns(self)
	vendor.Vendor:Debug("_UpdateDropDowns enter")
	-- stacksize
	local stackSize = math.min(self.item.stackCount, self.item.count);
	local stackSizes = self.stackSizes[stackSize];
	if (not stackSizes) then
		stackSizes = {};
		self.stackSizes[stackSize] = stackSizes;
		local i = 1;
		local prev = nil;
		while i <= stackSize do
			table.insert(stackSizes, i);
			prev = i;
			if (i < 5) then
				i = i + 1;
			else
				i = i * 2;
			end
		end
		if (prev and prev < stackSize) then
			table.insert(stackSizes, stackSize);
		end
	end
	if (self.item.preferedStackSize > 0) then
		self.stackDropDown:SetItems(stackSizes, self.item.preferedStackSize);
	else
		self.stackDropDown:SetItems(stackSizes, stackSize);
	end
	self.stackDropDown:SetRange(1, stackSize)
	-- stack count
	_UpdateCountDropDown(self);
	vendor.Vendor:Debug("_UpdateDropDowns exit")
end

--[[
	Shows information if mouse is over the selected item
--]]
local function _OnEnterItem(self, motion)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	if (self.ctrl.item) then
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
		GameTooltip.itemCount = self.ctrl.item.count
		GameTooltip:SetHyperlink(self.ctrl.item.link);
	else
		GameTooltip:SetText(L["Drag an item here to auction it"]);
	end
end

--[[
	Update the per item sell prizes and no. of stacks currently selected.
--]]
local function _UpdateSellInfo(self)
	local stackSize = self.stackDropDown:GetSelectedValue();
	self.minBidPerItem = MoneyInputFrame_GetCopper(self.startPriceBut);
	self.buyoutPerItem = MoneyInputFrame_GetCopper(self.buyoutPriceBut);
	local numStacks, items, _;
	local stackCount = self.countDropDown:GetSelectedValue();
	self.backlog = 0;
	if (type(stackCount) == "string") then
		-- we have a backdrop
		_, _, numStacks = string.find(stackCount, "^(%d+).*");
		self.backlog = self.item.count - (numStacks * stackSize);
		items = self.item.count;
	else
		numStacks = stackCount;
		items = numStacks * stackSize;
	end
	self.numStacks = numStacks;
	self.numItems = items;	
	if (self.db.profile.bidType == BID_TYPE_PER_ITEM) then
		-- nothing to be done
	elseif (self.db.profile.bidType == BID_TYPE_PER_STACK) then
		self.minBidPerItem = self.minBidPerItem / stackSize;
		self.buyoutPerItem = self.buyoutPerItem / stackSize;
	elseif (self.db.profile.bidType == BID_TYPE_ALL) then
		self.minBidPerItem = self.minBidPerItem / items;
		self.buyoutPerItem = self.buyoutPerItem / items;			
	end
	_UpdatePrices(self)
end

--[[
	Checks whether the current auction is valid and shows an error messages
	or enables the sell button accordingly.
--]]
local function _ValidateAuction(self)
	self.createBut:Show();
	self.createBut:Disable();
	self.buyoutErrorText:Hide();
	if (not self.item) then
		-- not item selected
		return;
	end
	local startPrize = MoneyInputFrame_GetCopper(self.startPriceBut);
	local buyoutPrize = MoneyInputFrame_GetCopper(self.buyoutPriceBut);
	-- Buyout price is less than the start price
	if (buyoutPrize > 0 and startPrize > buyoutPrize) then
		self.createBut:Hide();
		self.buyoutErrorText:Show();
		return;
	end
	-- Start price is 0 or greater than the max allowed
	if (startPrize < 1 or startPrize > MAXIMUM_BID_PRICE) then
		return;
	end
	_UpdateSellInfo(self);
	self.createBut:Enable();
end

--[[
	Auction duration radio button has been clicked
--]]
local function _DurationSelected(self, id)
	vendor.Vendor:Debug("duration selected");
	PlaySound("igMainMenuOptionCheckBoxOn");
	if (id == 1) then
		self.duration = DURATIONS[1];
		vendor.Vendor:Debug("short");
	elseif (id == 2) then
		self.duration = DURATIONS[2];
		vendor.Vendor:Debug("medium duration");
	else
		self.duration = DURATIONS[3];
		vendor.Vendor:Debug("long duration");
	end
	_UpdatePrices(self)
end

--[[
	Prizing model was selected.
--]]
local function _PrizingSelected(self, value)
	if (self.PRIZE_MODEL_CURRENT == value) then
		self.buyoutMul:SelectPriceModel(value)
		self.buyoutMul:Show()
	elseif (self.PRIZE_MODEL_MARKET == value) then
		self.buyoutMul:SelectPriceModel(value)
		self.buyoutMul:Show()
	elseif (self.PRIZE_MODEL_UNDERCUT == value) then
		self.buyoutMul:SelectPriceModel(value)
		self.buyoutMul:Show()
	elseif (self.PRIZE_MODEL_MULTIPLIER == value) then
		self.buyoutMul:SelectPriceModel(value)
		self.buyoutMul:Show()
	elseif (self.PRIZE_MODEL_LOWER == value) then
		self.buyoutMul:SelectPriceModel(value)
		self.buyoutMul:Show()
	else
		self.buyoutMul:Hide()
	end
	self:RefreshPrices()
end

--[[
	Prints the uptodateness of the currenlty selected scanset.
--]]
local function _PrintUpToDateness(self)
	local scanResult = self.itemModel:GetScanSet()
	if (scanResult and scanResult:Size() > 0) then
		-- 0 seconds lead to an empty string
		local secs = SecondsToTime(scanResult:GetUpToDateness() + 1)
		local msg = L["Up-to-dateness: <= "]..secs
		local itemLink = scanResult:GetItemLink()
		if (itemLink) then
			-- get averages
			local minBid, buyout = vendor.Statistic:GetCurrentAuctionInfo(itemLink, vendor.AuctionHouse:IsNeutral())
			local minBidMsg = "-"
			local buyoutMsg = "-"
			if (minBid and minBid > 0) then
				minBidMsg = vendor.Format.FormatMoney(minBid)
			end
			if (buyout and buyout > 0) then
				buyoutMsg = vendor.Format.FormatMoney(buyout)
			end
			if (GetLocale() == "zhTW" or GetLocale() == "zhCN" ) then
				msg = msg.." - ("..minBidMsg.."/"..buyoutMsg..")"
			else
				msg = msg.." - Ø("..minBidMsg.."/"..buyoutMsg..")"
			end
		end
		self.statusText:SetText(msg)
	else
		self.statusText:SetText("")
	end
end

--[[
	Selects the given itemLink for selling
--]]
local function _SelectItem(self, itemLink)
	vendor.Vendor:Debug("_SelectItem enter")
	if (not itemLink) then
		SalesFrameItemName:SetText("");
		SalesFrameItemCount:Hide();
		SalesFrameItem:SetNormalTexture(nil);
		self.lastItem = self.item;
		self.item = nil;
		self.stackDropDown:SetValidating(false)
		self.countDropDown:SetValidating(false)
	else
		self.stackDropDown:SetValidating(true)
		self.countDropDown:SetValidating(true)
		self.auto:SetItem(itemLink)
		local name, link, stackCount, texture, count = self.inventory:GetItemInfo(itemLink);
		if (name) then
			SalesFrameItem:SetNormalTexture(texture);
			SalesFrameItemName:SetText(name);
			if (count > 1) then
				SalesFrameItemCount:SetText(count);
				SalesFrameItemCount:Show();
			else
				SalesFrameItemCount:Hide();
			end
			local itemLinkKey = vendor.Items:GetItemLinkKey(itemLink);
			local preferedStackSize = 0;
			local itemInfo = vendor.Items:GetItemInfo(itemLinkKey, vendor.AuctionHouse:IsNeutral());
			if (itemInfo) then
				vendor.Vendor:Debug("found itemInfo")
				if (self.db.profile.rememberStackSize and itemInfo.stackSize <= count) then
					preferedStackSize = itemInfo.stackSize;			
				end
				if (self.db.profile.rememberDuration and itemInfo.duration > 0) then
					vendor.Vendor:Debug("select duration: "..itemInfo.duration)
					self.durationDropDown:SelectValue(itemInfo.duration)
					_DurationSelected(self, itemInfo.duration)
				else
					vendor.Vendor:Debug("select default duration: %s", self.db.profile.defaultDuration)
					vendor.Vendor:Debug("id: %d", self.db.profile.defaultDuration)
					self.durationDropDown:SelectValue(self.db.profile.defaultDuration)
					_DurationSelected(self, self.db.profile.defaultDuration)
				end
				if (self.db.profile.rememberPriceModel and itemInfo.priceModel > 0) then
					vendor.Vendor:Debug("select priceModel: "..itemInfo.priceModel)
					self.prizingDropDown:SelectValue(itemInfo.priceModel)
					_PrizingSelected(self, itemInfo.priceModel)
				end
			end
			self.item = {name = name, link = link, stackCount = stackCount, texture = texture, count = count, preferedStackSize = preferedStackSize, itemLinkKey = itemLinkKey};
			_UpdateDropDowns(self);
		else
			-- item disappaered after it had been selected
			_SelectItem(self, nil);
		end
	end
	self:RefreshPrices()
	_ValidateAuction(self);
	vendor.Vendor:Debug("_SelectItem exit")
end

--[[
	Picks the given item for selling.
--]]
local function _PickItem(self, itemLink)
	-- use blizzard's auction slot to calculate deposit and start prize 
	ClickAuctionSellItemButton();
	local _, _, count, _, _, price = GetAuctionSellItemInfo();
	self.deposits = {};
	for _, duration in ipairs(DURATIONS) do
		self.deposits[duration] = CalculateAuctionDeposit(duration) / count;
	end;
	self.startPrize = price / count;
	_SelectItem(self, itemLink);
	-- now free blizzard's auction slot
	ClickAuctionSellItemButton();
	ClearCursor();
	-- display all known auctions about the item
	local scanSet = vendor.Scanner:GetScanSet(vendor.AuctionHouse:IsNeutral(), itemLink);
	self.itemModel:SetScanSet(scanSet)
	self.itemTable:Show()
	_PrintUpToDateness(self)
	vendor.Vendor:Debug("Check ScanSet freshness and size");
	if (scanSet:Size() == 0 or (scanSet:GetUpToDateness() >= self.db.profile.autoRefreshTime)) then
		vendor.Vendor:Debug("Refresh ScanSet");
		_RefreshClicked(self);
	end
end

--[[
	Picks the currently dragged item for selling.
--]]
local function _PickItemFromCursor(self)
	local type, _, itemLink = GetCursorInfo();
	if (type and type == "item" and itemLink) then
		_PickItem(self, itemLink)
	end
end

--[[
	Updates the list of inventory items.
--]]
local function _UpdateInventory(self)
	if (self.frame and self.frame:IsVisible()) then
		self.inventory:Update();
		if (self.item) then
			_SelectItem(self, self.item.link);
		end
	 end
end

--[[
	Creates an auction with the currently selected item.
--]]
local function _CreateAuction(self)
	if (self.item) then
		local itemLink = self.item.link;
		local runTime = self.duration;
		local stackSize = self.stackDropDown:GetSelectedValue();
		
		if (not self.stackDropDown:IsValid() or not self.countDropDown:IsValid()) then
			vendor.Vendor:Error(L["Please correct the drop downs first!"])
			return nil
		end
		
		-- remember several dropdown button values for next time
  		local itemInfo = vendor.Items:GetItemInfo(self.item.itemLinkKey, vendor.AuctionHouse:IsNeutral());
		if (not itemInfo) then
			itemInfo = vendor.Items:CreateItemInfo();
		end
		if (self.selectedStackSize) then
			-- a stackSize was selected manually, remember it
			itemInfo.stackSize = self.selectedStackSize;
		end
		itemInfo.priceModel = self.prizingDropDown:GetSelectedValue()
		itemInfo.duration = self.durationDropDown:GetSelectedValue()
 		vendor.Items:PutItemInfo(self.item.itemLinkKey, vendor.AuctionHouse:IsNeutral(), itemInfo);
		self.selectedStackSize = nil;
		
		-- create sell task
		_UpdateSellInfo(self);
		local task = vendor.SellTask:new(self.inventory, itemLink, self.minBidPerItem, self.buyoutPerItem, runTime, stackSize, self.numStacks, self.backlog);
		vendor.TaskQueue:AddTask(task);
		
		-- clear focus of edit boxes
		self.stackDropDown:ClearFocus()
		self.countDropDown:ClearFocus()
	end
end

--[[
	Creates the auction buttons.
--]]
local function _CreateAuctionButtons(self)
	-- item to be sold
	local texture = vendor.GuiTools.CreateTexture(nil, self.frame, "ARTWORK", "Interface\\Addons\\AuctionMaster\\Media\\ItemContainer", 256, 64)
	texture:SetPoint("TOPLEFT", 27, -92)
	local f = self.frame:CreateFontString("SalesFrameItemText", "ARTWORK", "GameFontHighlightSmall");
	f:SetText(L["Auction Item"]);
	f:SetPoint("TOPLEFT", 28, -79);
	self.itemBut = CreateFrame("Button", "SalesFrameItem", self.frame);
	self.itemBut.ctrl = self; 
	self.itemBut:SetWidth(37);
	self.itemBut:SetHeight(37);
	self.itemBut:SetPoint("TOPLEFT", 28, -94);
	local f = self.itemBut:CreateFontString("SalesFrameItemName", "BACKGROUND", "GameFontNormal");
	f:SetWidth(124);
	f:SetHeight(30);
	f:SetPoint("TOPLEFT", self.itemBut, "TOPRIGHT", 5, 0);
	local f = self.itemBut:CreateFontString("SalesFrameItemCount", "OVERLAY", "NumberFontNormal");
	f:SetJustifyH("RIGHT");
	f:SetPoint("BOTTOMRIGHT", -5, 2);
	self.itemBut:RegisterForDrag("LeftButton");
	self.itemBut:SetScript("OnClick", function(but) _PickItemFromCursor(but.ctrl); _ValidateAuction(but.ctrl); end);
	self.itemBut:SetScript("OnDragStart", function(but) _PickItemFromCursor(but.ctrl); _ValidateAuction(but.ctrl); end);
	self.itemBut:SetScript("OnReceiveDrag", function(but) _PickItemFromCursor(but.ctrl); end);
	self.itemBut:SetScript("OnEnter", _OnEnterItem);
	self.itemBut:SetScript("OnLeave", function() GameTooltip:Hide(); end);
	self.itemBut:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square");

	-- duration of auction
	vendor.Vendor:Debug("create durations")
	self.durationDropDown = vendor.DropDownButton:new("SalesFrameDuration", self.frame, 85, L["Auction Duration"], "");
	self.durationDropDown:SetPoint("TOPLEFT", 10, -146);
 	self.durationDropDown:SetId(DURATION_DROPDOWN_ID);
	self.durationDropDown:SetListener(self);
    local durations = {};
	table.insert(durations, {value = 1, text = "12 "..GetText("HOURS", nil, 12)});
	table.insert(durations, {value = 2, text = "24 "..GetText("HOURS", nil, 24)});
	table.insert(durations, {value = 3, text = "48 "..GetText("HOURS", nil, 48)});
	self.durationDropDown:SetItems(durations, self.db.profile.defaultDuration)
	self.duration = DURATIONS[self.db.profile.defaultDuration]

	-- stacksize
	self.stackDropDown = vendor.DropDownButton:new("SalesFrameStackSize", self.frame, 50, L["Stack size"], L["Selects the size of the stacks to be sold."]);
	self.stackDropDown:SetPoint("TOPLEFT", 118, -146);
	self.stackDropDown:SetId(STACK_DROPDOWN_ID);
	self.stackDropDown:SetListener(self);
	self.stackDropDown:SetEditable(true)
	self.stackDropDown:SetNumeric(true)
	
	-- count of "stacks"
	self.countDropDown = vendor.DropDownButton:new("SalesFrameCount", self.frame, 50, L["Amount"], L["Selects the number of stacks to be sold.\nThe number with the +-suffix sells\nalso any remaining items."]);
	self.countDropDown:SetPoint("TOPLEFT", SalesFrameStackSize, "BOTTOMLEFT", 0, -13);
 	self.countDropDown:SetId(COUNT_DROPDOWN_ID);
	self.countDropDown:SetListener(self);
	self.countDropDown:SetEditable(true)
	self.countDropDown:SetNumeric(true)
	
	-- prizing model
	self.prizingDropDown = vendor.DropDownButton:new("SalesFramePrizing", self.frame, 85, L["Price calculation"], L["Selects the mode for calculating the sell prices.\nThe default mode Fixed price just select the last sell price."]);
	self.prizingDropDown:SetPoint("TOPLEFT", SalesFrameDuration, "BOTTOMLEFT", 0, -13);
 	self.prizingDropDown:SetId(PRIZING_DROPDOWN_ID);
	self.prizingDropDown:SetListener(self);
    local prizeModels = {};
	table.insert(prizeModels, {value = self.PRIZE_MODEL_FIX, text = L["Fixed price"]});
	table.insert(prizeModels, {value = self.PRIZE_MODEL_CURRENT, text = L["Current price"]});
	table.insert(prizeModels, {value = self.PRIZE_MODEL_UNDERCUT, text = L["Undercut"]})
	table.insert(prizeModels, {value = self.PRIZE_MODEL_LOWER, text = L["Lower market threshold"]})
	table.insert(prizeModels, {value = self.PRIZE_MODEL_MULTIPLIER, text = L["Selling price"]})
	table.insert(prizeModels, {value = self.PRIZE_MODEL_MARKET, text = L["Market price"]});
	self.prizingDropDown:SetItems(prizeModels, self.PRIZE_MODEL_FIX);
		
	self.buyoutMul = vendor.BuyoutModifier:new(self.frame, self.db.profile.buyoutMod)
	self.buyoutMul:SetPoint("TOPLEFT", SalesFramePrizing, "BOTTOMLEFT", 24, -6)
	
	self.auto:CreateFrame(self.frame)
	
	-- hrule to separate the sections
	local texture = vendor.GuiTools.CreateTexture(nil, self.frame, "ARTWORK", "Interface\\Addons\\AuctionMaster\\Media\\HRule", 256, 16)
	texture:SetPoint("TOPLEFT", 23, -260)
	
	-- starting prize
	f = self.frame:CreateFontString("SalesFramePriceText", "ARTWORK", "GameFontHighlightSmall");
	f:SetPoint("TOPLEFT", 28, -271);
	self.minBidFont = f
	self.startPriceBut = CreateFrame("Frame", "SalesFrameStartPrice", self.frame, "MoneyInputFrameTemplate");
	self.startPriceBut:SetPoint("TOPLEFT", SalesFramePriceText, "BOTTOMLEFT", 3, -2);
	self.startPriceBut.controller = self;

	local frameName = self.startPriceBut:GetName();
	local goldBut = getglobal(frameName.."GoldButton");
	MoneyInputFrame_SetOnValueChangedFunc(self.startPriceBut, function() _ValidateAuction(self) end);
	SalesFrameStartPriceGold:SetMaxLetters(6);
	
	-- buyout prize
	f = self.frame:CreateFontString("SalesFrameBuyoutText", "ARTWORK", "GameFontHighlightSmall");
	self.buyoutFont = f
	f:SetPoint("TOPLEFT", SalesFramePriceText, 0, -35);
	self.buyoutPriceBut = CreateFrame("Frame", "SalesFrameBuyoutPrice", self.frame, "MoneyInputFrameTemplate");
	self.buyoutPriceBut:SetPoint("TOPLEFT", 31, -317);
	self.buyoutPriceBut.controller = self;
	MoneyInputFrame_SetOnValueChangedFunc(self.buyoutPriceBut, function() _ValidateAuction(self) end);
	SalesFrameBuyoutPriceGold:SetMaxLetters(6);

	-- hrule to separate the sections
	local texture = vendor.GuiTools.CreateTexture(nil, self.frame, "ARTWORK", "Interface\\Addons\\AuctionMaster\\Media\\HRule", 256, 16)
	texture:SetPoint("TOPLEFT", 23, -338)
	
	-- area for sell information
	self.sellInfo = vendor.SellInfo:new(self.frame)
	self.sellInfo:SetPoint("TOPLEFT", 25, -347)
	
	-- Set focus rules
	self.stackDropDown:SetPrevFocus(SalesFrameBuyoutPriceCopper)
	self.stackDropDown:SetNextFocus(self.countDropDown:GetEditBox())
	self.countDropDown:SetPrevFocus(self.stackDropDown:GetEditBox())
	self.countDropDown:SetNextFocus(SalesFrameStartPriceGold)
	MoneyInputFrame_SetPreviousFocus(SalesFrameStartPrice, self.countDropDown:GetEditBox())
	MoneyInputFrame_SetNextFocus(SalesFrameStartPrice, SalesFrameBuyoutPriceGold)
	MoneyInputFrame_SetPreviousFocus(SalesFrameBuyoutPrice, SalesFrameStartPriceCopper)
	MoneyInputFrame_SetNextFocus(SalesFrameBuyoutPrice, self.stackDropDown:GetEditBox())
	
	-- total deposit
	f = self.frame:CreateFontString("SalesFrameDepositText", "ARTWORK", "GameFontNormal");
	f:SetText(L["Deposit:"]);
	f:SetPoint("TOPLEFT", 28, -364);
	f:Hide()
	self.depositBut = CreateFrame("Frame", "SalesFrameDeposit", self.frame, "DepositFrameTemplate");
	self.depositBut:SetPoint("LEFT", SalesFrameDepositText, "RIGHT", 5, 0);
	self.depositBut.controller = self;
	self.depositBut:Hide()
	
	-- up-to-dateness of the item list
	f = self.frame:CreateFontString("SalesFrameUpToDatenessText", "ARTWORK", "GameFontNormal");
	f:SetText("");
	f:SetPoint("BOTTOMLEFT", 300, 20);
	self.statusText = f;
	
	-- auction creation
	self.createBut = CreateFrame("Button", "SalesFrameCreate", self.frame, "UIPanelButtonTemplate");
	self.createBut:SetText(L["Create Auction"]);
	self.createBut:SetWidth(191);
	self.createBut:SetHeight(20);
	self.createBut:SetPoint("BOTTOMLEFT", 18, 39);
	self.createBut.ctrl = self;
	self.createBut:SetScript("OnClick", function(but) _CreateAuction(but.ctrl) end);
	
	-- auction creation not possible error text
	self.buyoutErrorText = self.frame:CreateFontString("SalesFrameBuyoutErrorText", "ARTWORK", "GameFontRedSmall");
	self.buyoutErrorText:SetText(L["Buyout < bid"]);
	self.buyoutErrorText:SetPoint("TOPLEFT", self.createBut, "TOPLEFT", 15, -5);
	
	_UpdateFonts(self)
end

--[[
	Creates the central close etc. buttons.
--]]
local function _CreateFrameButtons(self)
	self.closeBut = vendor.AuctionHouse:CreateCloseButton(self.frame, "SalesFrameClose");
end

--[[
	Updates according to current settings.
--]]
local function _ApplySettings(self)
	if (self.db.profile.pickupByClick and not self:IsHooked("ContainerFrameItemButton_OnModifiedClick")) then
		self:RawHook("ContainerFrameItemButton_OnModifiedClick", true)
	end
	if (not self.db.profile.pickupByClick and self:IsHooked("ContainerFrameItemButton_OnModifiedClick")) then
		self:Unhook("ContainerFrameItemButton_OnModifiedClick")
	end
	if (self.item) then
		-- change prizes according to stacksize/numItems
--		if (self.db.profile.bidType == BID_TYPE_PER_ITEM) then
--			_SetPrizes(self, self.minBidPerItem, self.buyoutPerItem);
--		elseif (self.db.profile.bidType == BID_TYPE_PER_STACK) then
--			local stackSize = self.stackDropDown:GetSelectedValue()
--			_SetPrizes(self, self.minBidPerItem * stackSize, self.buyoutPerItem * stackSize);
--		elseif (self.db.profile.bidType == BID_TYPE_ALL) then
--			_SetPrizes(self, self.minBidPerItem * self.numItems, self.buyoutPerItem * self.numItems);
--		end
		_UpdatePrices(self)
	end
	if (self.db.profile.bidCalc) then
		vendor.Vendor.options2.args.seller.args.bidMul.disabled = true
	else
		vendor.Vendor.options2.args.seller.args.bidMul.disabled = nil
	end
	_UpdateFonts(self)
	if (self.item) then
		self:RefreshPrices()
	end
end

--[[
	Sets the given value in the profile.
--]]
local function _SetValue(info, value)
	self.db.profile[info.arg] = value;
	_ApplySettings(self)
end

--[[
	Returns the given value from the profile.
--]]
local function _GetValue(info)
	return self.db.profile[info.arg]
end

--[[
	Options for this module.
--]]
local function _CreateOptions()
	vendor.Vendor.options2.args.seller = {
		type = "group",
		name = L["Seller"],
		desc = L["Seller"],
		args = {
			autoMode ={
				type = "group",
				name = L["Auto selling"],
				desc = L["Settings for automatically selecting the best fitting price model."],
				order = 1,
				args = {
					aboveMarketThreshold = {
						type = "range",
        				name = L["Upper market threshold"],
        				desc = L["Minimal needed percentage of buyouts compared to market price, until they are assumed to be considerably above the market price."],
        				min = 101,
        				max = 500,
        				step = 1,
        				get = _GetValue,
        				set = _SetValue,
        				arg = "upperMarketThreshold",
        				order = 1,
        			},
					underMarketThreshold = {
						type = "range",
        				name = L["Lower market threshold"],
        				desc = L["Maximal allowed percentage of buyouts compared to market price, until they are assumed to be considerably under the market price."],
        				min = 1,
        				max = 99,
        				step = 1,
        				get = _GetValue,
        				set = _SetValue,
        				arg = "lowerMarketThreshold",
        				order = 10,
        			},
        		},
        	},
			autoRefreshTime = {
				type = "range",
				name = L["Autorefresh time"],
				desc = L["Selects the minimum time in seconds that has to be passed, before the auction house is automatically scanned for the item to be sold."],
				min = 10,
				max = 360,
				step = 1,
				get = _GetValue,
				set = _SetValue,
				arg = "autoRefreshTime",
				order = 10,
			},
			pickupByClick = {
				type = "toggle",
				name = L["Pickup by click"],
				desc = L["Pickup items to be soled, when they are shift left clicked."],
				get = _GetValue,
				set = _SetValue,
				arg = "pickupByClick",
				order = 20,
			},
			rememberStackSize = {
				type = "toggle",
				name = L["Remember stacksize"],
				desc = L["Automatically selects the stacksize used the last time for a given item."],
				get = _GetValue,
				set = _SetValue,
				arg = "rememberStackSize",
				order = 30,
			},
			rememberDuration = {
				type = "toggle",
				name = L["Remember duration"],
				desc = L["Automatically selects the duration used the last time for a given item."],
				get = _GetValue,
				set = _SetValue,
				arg = "rememberDuration",
				order = 40,
			},
			defaultDuration = {
				type = "select",
				name = L["Default duration"],
				desc = L["Default duration for creating auctions."],
				get = _GetValue,
				set = _SetValue,
				values = {"12h", "24h", "48h"}, 
				arg = "defaultDuration",
				order = 50,
			},																		
			rememberPriceModel = {
				type = "toggle",
				name = L["Remember price model"],
				desc = L["Automatically selects the price model used the last time for a given item."],
				get = _GetValue,
				set = _SetValue,
				arg = "rememberPriceModel",
				order = 60,
			},
			bidType = {
				type = "select",
				name = L["Bid type"],
				desc = L["Selects which prices should be shown in the bid and buyout input fields."],
				get = _GetValue,
				set = _SetValue,
				values = BID_TYPES, 
				arg = "bidType",
				order = 70,
			},
			bidCalcDesc = {
				type = "description",
				order = 72,
				name = L["Should the starting price be calculated? Otherwise it is dependant from the buyout price."],
			},
			bidCalc = {
				type = "toggle",
				name = L["Calculate starting price"],
				desc = L["Should the starting price be calculated? Otherwise it is dependant from the buyout price."],
				get = _GetValue,
				set = _SetValue,
				arg = "bidCalc",
				order = 75,
			},																		
			bidMul = {
				type = "range",
				name = L["Bid multiplier"],
				desc = L["Selects the percentage of the buyout price the bid value should be set to. A value of 100 will set it to the equal value as the buyout price. It will never fall under blizzard's suggested starting price, which is based on the vendor selling value of the item."],
				min = 1,
				max = 100,
				step = 1,
				get = _GetValue,
				set = _SetValue,
				arg = "bidMul",
				order = 80,
			},
		}
	}
	vendor.Vendor.AceConfigRegistry:RegisterOptionsTable("AuctionMaster Seller", vendor.Vendor.options2.args.seller)
	vendor.Vendor.AceConfigDialog:AddToBlizOptions("AuctionMaster Seller", L["Seller"], "AuctionMaster")
end

--[[
	Initializes the module.
--]]
function vendor.Seller:OnInitialize()
	self.db = vendor.Vendor.db:RegisterNamespace("Seller", {
		profile = {
    		autoRefreshTime = 60,
    		pickupByClick = true,
    		rememberStackSize = true,
    		rememberDuration = true,
    		rememberPriceModel = true,
    		upperMarketThreshold = 200,
    		lowerMarketThreshold = 30,
    		autoPriceModel = true,		
    		defaultDuration = 3,
    		bidType = 1,
    		itemTableCfg = vendor.ItemTable.CreateConfig(vendor.ScanSetItemModel.NAME, vendor.ScanSetItemModel.COUNT, vendor.ScanSetItemModel.OWNER, vendor.ScanSetItemModel.BID, vendor.ScanSetItemModel.BUYOUT),
    		bidCalc = true,
    		bidMul = 90,
    	}
	});
	_MigrateDb(self)
	_CreateOptions();
	self.auto = vendor.AutomaticPriceModel:new()	
end

--[[
	Initializes the module.
--]]
function vendor.Seller:OnEnable()
	_ApplySettings(self)
	self.playerName = UnitName("player")
	self.isScanning = false;
	self.stackSizes = {};
	self.inventory = vendor.InventoryHandle:new();
	self.lastItem = nil;
	self:RegisterEvent("BAG_UPDATE");
	self:RegisterEvent("AUCTION_HOUSE_CLOSED");
	self:RegisterEvent("NEW_AUCTION_UPDATE");
   	self:Hook("StartAuction", true);
	-- register for new ScanResults
	vendor.Scanner:AddScanSnapshotListener(self);
	self.frame = vendor.AuctionHouse:CreateTabFrame("AuctionFrameVendor", "AuctionMaster", L["Sell"], self);
	--self.vendorTabButton = vendor.AuctionHouse:CreateTabButton(L["Seller"], 4);
	_CreateFrameButtons(self);
	_CreateAuctionButtons(self);
	local itemModel = vendor.ScanSetItemModel:new()
	self.itemModel = itemModel
	local cmds = vendor.Vendor:OrderTable(CMDS)
	local itemTable = vendor.ItemTable:new(self.frame, itemModel, cmds, self.db.profile.itemTableCfg, 609, 339)
	itemTable:SetPoint("TOPLEFT", 214, -70)
	self.itemTable = itemTable
end

--[[
	Updates the gui for displaying the frame (Interface method).
--]]
function vendor.Seller:UpdateTabFrame()
	vendor.Vendor:Debug("Seller:UpdateTabFrame enter")
	AuctionFrameTopLeft:SetTexture("Interface\\Addons\\AuctionMaster\\Media\\UI-AuctionFrame-Auction-TopLeft");
	AuctionFrameTop:SetTexture("Interface\\AuctionFrame\\UI-AuctionFrame-Auction-Top");
	AuctionFrameTopRight:SetTexture("Interface\\AuctionFrame\\UI-AuctionFrame-Auction-TopRight");
	AuctionFrameBotLeft:SetTexture("Interface\\Addons\\AuctionMaster\\Media\\UI-AuctionFrame-Auction-BotLeft");
	AuctionFrameBot:SetTexture("Interface\\AuctionFrame\\UI-AuctionFrame-Auction-Bot");
	AuctionFrameBotRight:SetTexture("Interface\\Addons\\AuctionMaster\\Media\\UI-AuctionFrame-Auction-BotRight");
	self.auto:SetItem(nil)
	vendor.Vendor:Debug("Seller:UpdateTabFrame exit")
end

--[[
	Returns the type of this auction house tab.
--]]
function vendor.Seller:GetTabType()
	return SELLER_TAB;
end
	
--[[
	Shows the tabbed frame (Interface method).
--]]
function vendor.Seller:ShowTabFrame()
	vendor.Vendor:Debug("Seller:ShowTabFrame enter")
	self.frame:Show();
	_UpdateInventory(self);
	self.itemTable:Show()
	vendor.Vendor:Debug("Seller:ShowTabFrame exit")
end

--[[
	Hides the tabbed frame (Interface method).
--]]
function vendor.Seller:HideTabFrame()
	vendor.Vendor:Debug("Seller:HideTabFrame enter")
	self.frame:Hide();
	vendor.Vendor:Debug("Seller:HideTabFrame exit")
end

--[[
	Interface method for DropDownButton.
--]]
function vendor.Seller:DropDownButtonSelected(button, value)
	vendor.Vendor:Debug("dropdown selected");
	local id = button:GetId();
	vendor.Vendor:Debug("id: "..id);	
	if (id == DURATION_DROPDOWN_ID) then
		vendor.Vendor:Debug("duration selected");
		-- duration has been selected
		_DurationSelected(self, value)
	elseif (id == PRIZING_DROPDOWN_ID) then
		vendor.Vendor:Debug("prizing selected");
		_PrizingSelected(self, value)
	elseif (self.item) then
		local stackSize = self.stackDropDown:GetSelectedValue();
		if (id == STACK_DROPDOWN_ID) then
			vendor.Vendor:Debug("stack selected");
			-- new stacksize has been selected
			_UpdateCountDropDown(self);
			self.selectedStackSize = stackSize;
			-- only change the prizes, if they are shown per stack
--			if (self.db.profile.bidType == BID_TYPE_PER_STACK) then
--				_SetPrizes(self, self.minBidPerItem * stackSize, self.buyoutPerItem * stackSize);
--			end
		elseif (id == COUNT_DROPDOWN_ID) then
			vendor.Vendor:Debug("count selected");
			-- change only needed if bid type is set to all
--			if (self.db.profile.bidType == BID_TYPE_ALL) then
--				_SetPrizes(self, self.minBidPerItem * self.numItems, self.buyoutPerItem * self.numItems);
--			end
		end
		--_UpdatePrices(self)
		self:RefreshPrices()
	end
end

--[[
	BAG_UPDATe event has been triggered.
--]]
function vendor.Seller:BAG_UPDATE()
	_UpdateInventory(self);
end

--[[
	Auction house has been closed
--]]
function vendor.Seller:AUCTION_HOUSE_CLOSED()
	_SelectItem(self, nil);
end

--[[
	Hooks the StartAuction function to remember the prizes.
--]]
function vendor.Seller:StartAuction(minBid, buyoutPrize, runTime)
   	-- we want to remember the "minBid" and "buyoutPrice" for the next time
   	local name, texture, count, quality, canUse, price = GetAuctionSellItemInfo();
   	if (name) then
      	-- unfortunately we need the itemLink, we have to scan the inventory
      	local itemLink = _FindInventoryItemLink(name);
      	if (itemLink) then
      		local itemLinkKey = vendor.Items:GetItemLinkKey(itemLink);
      		local itemInfo = vendor.Items:GetItemInfo(itemLinkKey, vendor.AuctionHouse:IsNeutral());
			if (not itemInfo) then
				itemInfo = vendor.Items:CreateItemInfo();
			end
	 		itemInfo.minBid = math.floor(minBid / count);
	 		itemInfo.buyout = math.floor(buyoutPrize / count);
	 		vendor.Items:PutItemInfo(itemLinkKey, vendor.AuctionHouse:IsNeutral(), itemInfo);
      	end
   	end
   	--self.hooks.StartAuction(minBid, buyoutPrize, runTime);
end

--[[
	Event callback for filling in last auction prizes for the given item.
--]]
function vendor.Seller:NEW_AUCTION_UPDATE()
	MoneyInputFrame_SetCopper(StartPrice, 0);
	MoneyInputFrame_SetCopper(BuyoutPrice, 0);
	-- we want to fill in old "minBid" and "buyoutPrice"
	local name, texture, count, quality, canUse, price = GetAuctionSellItemInfo();
	if (name) then
		-- unfortunately we need the itemLink, we have to scan the inventory
		local itemLink = _FindInventoryItemLink(name);
		if (itemLink) then
			local itemInfo = vendor.Items:GetItemInfo(itemLinkKey, vendor.AuctionHouse:IsNeutral());
			if (itemInfo) then
				if (itemInfo.minBid > 0) then
					MoneyInputFrame_SetCopper(StartPrice, itemInfo.minBid * count);
	 			end
	 			if (itemInfo.buyout > 0) then
	    			MoneyInputFrame_SetCopper(BuyoutPrice, itemInfo.buyout * count);
	 			end
      		end
      	end
   	end
end

--[[
	Returns information about the currently selected item, if any.
	@return itemName, itemLink
--]]
function vendor.Seller:GetSelectedItemInfo()
	if (self.item) then
		return self.item.name, self.item.link;
	end
	return nil;
end	

--[[
	ScanResultListener interface method for new or updated
	Scanresults.
--]]
function vendor.Seller:ScanSnapshotUpdated(scanSnapshot, isNeutral)
	vendor.Vendor:Debug("vendor.Seller:ScanSnapshotUpdated enter");
	local prizeModel = self.prizingDropDown:GetSelectedValue();
	if (prizeModel ~= self.PRIZE_MODEL_FIX) then
		self.auto:Update()
		self:RefreshPrices()
	end
	local scanSet = self.itemModel:GetScanSet()
	if (scanSet) then
		self.itemModel:SetScanSet(vendor.Scanner:GetScanSet(scanSet:IsNeutral(), scanSet:GetItemLink()))
		self.itemTable:Show()
	end
	_PrintUpToDateness(self)
	vendor.Vendor:Debug("vendor.Seller:ScanSnapshotUpdated exit");
end

--[[
	Returns whether the seller frame is currently visible.
--]]
function vendor.Seller:IsSellerFrameVisible()
	return vendor.AuctionHouse:IsAuctionHouseTabShown(SELLER_TAB)
end

--[[
	Inventory item was clicked, we pick it up for the seller frame, if appropriate. 
--]]
function vendor.Seller:ContainerFrameItemButton_OnModifiedClick(frame, button, ...)
	vendor.Vendor:Debug("ContainerFrameItemButton_OnModifiedClick")
	-- some addons forget to pass the button parameter :-(
	if ((not button or button == "LeftButton") and IsShiftKeyDown() and self:IsSellerFrameVisible() and not CursorHasItem()) then
		vendor.Vendor:Debug("pickup item")
		local itemLink = GetContainerItemLink(frame:GetParent():GetID(), frame:GetID())
		if (itemLink) then
			-- we have to pickup for deposit calculation
			PickupContainerItem(frame:GetParent():GetID(), frame:GetID())
			_PickItem(self, itemLink)
		else
			vendor.Vendor:Debug("no itemlink")
		end
	else
		vendor.Vendor:Debug("don't pick item'")
		self.hooks["ContainerFrameItemButton_OnModifiedClick"](frame, button, ...)
	end
end

--[[
	Selects the given pricing model.
--]]
function vendor.Seller:SelectPricingModel(id)
	vendor.Vendor:Debug("Seller:SelectPricingModel enter id: "..(id or "NA"))
	self.prizingDropDown:SelectValue(id)
	_PrizingSelected(self, id)
	vendor.Vendor:Debug("Seller:SelectPricingModel exit")
end

--[[
	Refreshes the prices for minbid resp. buyout. 
--]]
function vendor.Seller:RefreshPrices()
	vendor.Vendor:Debug("RefreshPrizes");
	local minBid = 0;
	local buyout = 0;
	local mult = 1;
	if (not self.stackDropDown:IsValid() or not self.countDropDown:IsValid()) then
		log:Debug("RefreshPrizes exit because of invalid drop downs")
		return nil
	end
	if (self.item) then
		-- determine the multiplier for the displayed prize
		if (self.db.profile.bidType == BID_TYPE_PER_STACK) then
			mult = self.stackDropDown:GetSelectedValue();
		elseif (self.db.profile.bidType == BID_TYPE_ALL) then
			local count = self.countDropDown:GetSelectedValue();
			if (type(count) == "string") then
				-- sell all
				mult = self.item.count;
			else
				mult = self.stackDropDown:GetSelectedValue() * count;
			end
		end
		-- determine prizing model
		local prizeModel = self.prizingDropDown:GetSelectedValue();
		if (prizeModel == self.PRIZE_MODEL_FIX) then
			minBid, buyout = _GetItemAuctionSellValues(self.item.link);
		elseif (prizeModel == self.PRIZE_MODEL_MARKET) then
			minBid, buyout = _GetMarketPrice(self, self.item.link);
		elseif (prizeModel == self.PRIZE_MODEL_CURRENT) then
			minBid, buyout = _GetCurrentPrice(self, self.item.link);
		elseif (prizeModel == self.PRIZE_MODEL_UNDERCUT) then
			minBid, buyout = _GetUndercutPrice(self, self.item.link);
		elseif (prizeModel == self.PRIZE_MODEL_MULTIPLIER) then
			minBid, buyout = _GetMultiplierPrice(self, self.item.link);
		elseif (prizeModel == self.PRIZE_MODEL_LOWER) then
			minBid, buyout = _GetLowerPrice(self, self.item.link);
		end
		if (not minBid or minBid <= 0) then
			minBid = self.startPrize;
		end
		if (not buyout or buyout <= 0) then
			buyout = minBid * 2
		end
	end
	--_SetPrizes(self, minBid * mult, (buyout or 0) * mult);
	self.minBidPerItem = minBid
	self.buyoutPerItem = buyout or 0
	_UpdatePrices(self)
end
